@@ -9,6 +9,9 @@ import signal as _signal_
 # the spawn object class (for inputs).
 class Spawn(objects.Object):
 	def __init__(self,
+		#
+		# Should be initialized with function: syst3m.console.execute
+		#
 		# the full command (str) (#1).
 		command="ls",
 		# asynchronous.
@@ -42,7 +45,7 @@ class Spawn(objects.Object):
 			return r3sponse.error(f"Failed to spawn {self.response_str}, error: {e}.")
 		# check exit status.
 		if self.child.exitstatus not in [0, None]:
-			return Output(error=f"{self.response_str} returned exit status: [{spawn.child.exitstatus}].")
+			return OutputObject(error=f"{self.response_str} returned exit status: [{spawn.child.exitstatus}].")
 		self.__output__ = ""
 		return r3sponse.success(f"Successfully spawned {self.response_str}.")
 		#
@@ -267,7 +270,12 @@ class Spawn(objects.Object):
 			
 		# version 2.
 		# more stable.
-		return self.read(wait=True, timeout=None)
+		response = self.read(wait=True, timeout=timeout)
+		if response.success:
+			while True:
+				if len(response.output) > 0 and response.output[len(response.output)-1] == "\n": response.output = response.output[:-1]
+				else: break
+		return response
 
 		"""
 		# version 1.
@@ -294,13 +302,21 @@ class Spawn(objects.Object):
 
 		#
 
+	# check if process is crashed.
+	def crashed(self):
+		response = self.read(wait=False)
+		if not response.success:  return response
+		if self.exit_status not in [0, None]:
+			return r3sponse.error(f"{self.response_str} returned exit status: [{self.exit_status}].")
+		return r3sponse.success(f"{self.response_str} is not crashed.")
+
 	# is expecting input.
 	@property
 	def expecting(self):
 
 		# version 2.
 		if self.exit_status not in [None]: return False
-		elif self.closed: return False
+		elif self.child.closed: return False
 		elif not self.running: return False
 		else:
 
@@ -342,17 +358,6 @@ class Spawn(objects.Object):
 		boolean = self.child.isalive()
 		return boolean
 		#
-	# is closed.
-	@property
-	def closed(self):
-		"""if self.async_:
-			boolean = yield from self.child.closed
-		else:
-			boolean = self.child.closed
-		"""
-		boolean = self.child.closed
-		return boolean
-		#
 	# exit status.
 	@property
 	def exit_status(self):
@@ -372,9 +377,13 @@ class Spawn(objects.Object):
 
 	#
 
-# the output object class (very similair to ResonseObject).
-class Output(ResponseObject):
+# the output object class (very similair to the ResonseObject).
+class OutputObject(ResponseObject):
 	def __init__(self, 
+		#
+		# The return object from function: syst3m.console.execute
+		# The OutputObject object is very similair to the ResponseObject.
+		#
 		# the success message (param #1).
 		message=None,
 		# the attributes (param #2).
@@ -446,7 +455,7 @@ class Output(ResponseObject):
 
 	# object instance.
 	def instance(self):
-		return "Output"
+		return "OutputObject"
 		#
 	# assign to ResponseObject.
 	def response(self):
@@ -459,7 +468,7 @@ class Output(ResponseObject):
 # execute.
 def execute(
 	# Notes:
-	#   returns a syst3m.console.Output object (very similair to ResponseObject).
+	#   returns a syst3m.console.OutputObject object (very similair to ResponseObject).
 	#
 	# Mode:
 	#   option 1:
@@ -544,164 +553,161 @@ def execute(
 	
 	# start.
 	response = spawn.start()
-	if not response.success: return Output(error=f"Failed to start {response_str}, error: {response.error}")
+	if not response.success: return OutputObject(error=f"Failed to start {response_str}, error: {response.error}")
 	
-	# check exit status.
-	if spawn.child.exitstatus not in [0, None]:
-		return Output(error=f"{response_str} returned exit status: [{spawn.child.exitstatus}].")
+	# check crashed
+	response = spawn.crashed()
+	if not response.success:  return OutputObject(error=response.error)
 
-	# await.
-	for i in range(10):
-		if spawn.running: break
-		time.sleep(1)
-	if not spawn.running: return Output(error=f"Spawned {response_str} is not running.")
+	# has already exited.
+	elif spawn.exit_status in [0]:
+		
+		# get output.
+		response = spawn.wait(timeout=1.0)
+		if not response.success: return OutputObject(error=response.error) # exit status.
+		output = response.output
 
-	# str input.
-	if isinstance(input, (list, Array)):
-		if len(input) > 0:
-			str_input = Array(list(input[0].keys())).string(joiner=", ")
-		else:
-			str_input = Array(input).string(joiner=", ")
-	elif isinstance(input, (dict, Dictionary)):
-		str_input = Array(list(input.keys())).string(joiner=", ")
-	else: 
-		str_input = f"[{input}]"
+		#
 
-	# send optional input.
-	error_end_of_file = None
-	if not async_ and isinstance(input, (list, Array, dict, Dictionary)) and len(input) > 0:
+	# proceed if already finished.
+	elif spawn.exit_status not in [0]:
 
-		# expect one of the keys in the dictionary.
-		def process_dict_input(dictionary):
+		# await.
+		for i in range(10):
+			if spawn.running: break
+			time.sleep(1)
+		if not spawn.running: return OutputObject(error=f"Unable to start {response_str}.")
 
-			# vars.
-			expect = list(dictionary.keys())
-			send = list(dictionary.values())
-
-			# expect .
-			if defaults.options.log_level >= 8:
-				print(f"Expecting one of the following inputs: {expect}.")
-			response = spawn.expect(expect=expect, send=send, timeout=timeout)
-			if not response.success:
-				if "None of the specified inputs were expected." in response.error:
-					if optional:
-						return r3sponse.error(f"Unable to find the expected input but still success since it is optional.")
-					else:
-						return r3sponse.error(f"Specified input [{Array(expect).string(joiner=', ')}] was not expected.")
-				else: return response
-			if defaults.options.log_level >= 8:
-				print("Send response message:",response.message)
-
-			# success.
-			return r3sponse.success("Success.")
-
-		""" check expecting. """
-		if not spawn.expecting:
-			if defaults.options.log_level >= 1:
-				return Output(error=f"Failed to send {str_input} to {response_str}, child is not expecting any input [{spawn.child}].")
-			else:
-				return Output(error=f"Failed to send {str_input} to {response_str}, child is not expecting any input.")
-
-		# send all dicts in the list (list instance).
-		error_end_of_file = False
+		# str input.
 		if isinstance(input, (list, Array)):
-			for _input_ in input:
-				response = process_dict_input(_input_)
+			if len(input) > 0:
+				str_input = Array(list(input[0].keys())).string(joiner=", ")
+			else:
+				str_input = Array(input).string(joiner=", ")
+		elif isinstance(input, (dict, Dictionary)):
+			str_input = Array(list(input.keys())).string(joiner=", ")
+		else: 
+			str_input = f"[{input}]"
+
+		# send optional input.
+		error_end_of_file = None
+		if not async_ and isinstance(input, (list, Array, dict, Dictionary)) and len(input) > 0:
+
+			# expect one of the keys in the dictionary.
+			def process_dict_input(dictionary):
+
+				# vars.
+				expect = list(dictionary.keys())
+				send = list(dictionary.values())
+
+				# expect .
+				if defaults.options.log_level >= 8:
+					print(f"Expecting one of the following inputs: {expect}.")
+				response = spawn.expect(expect=expect, send=send, timeout=timeout)
+				if not response.success:
+					if "None of the specified inputs were expected." in response.error:
+						if optional:
+							return r3sponse.error(f"Unable to find the expected input but still success since it is optional.")
+						else:
+							return r3sponse.error(f"Specified input [{Array(expect).string(joiner=', ')}] was not expected.")
+					else: return response
+				if defaults.options.log_level >= 8:
+					print("Send response message:",response.message)
+
+				# success.
+				return r3sponse.success("Success.")
+
+			""" check expecting. """
+			if not spawn.expecting:
+				if defaults.options.log_level >= 1:
+					return OutputObject(error=f"Failed to send {str_input} to {response_str}, child is not expecting any input [{spawn.child}].")
+				else:
+					return OutputObject(error=f"Failed to send {str_input} to {response_str}, child is not expecting any input.")
+
+			# send all dicts in the list (list instance).
+			error_end_of_file = False
+			if isinstance(input, (list, Array)):
+				for _input_ in input:
+					response = process_dict_input(_input_)
+					if not response.success: 
+						if "End of file" in response.error:
+							error_end_of_file = True
+						else: 
+							# str input.
+							if isinstance(_input_, (list, Array)):
+								str_input = Array(_input_).string(joiner=", ")
+							elif isinstance(_input_, (dict, Dictionary)):
+								str_input = Array(list(_input_.keys())).string(joiner=", ")
+							else: 
+								str_input = f"[{_input_}]"
+							return OutputObject(error=f"Failed to send one of the expected input(s) [{str_input}], error: {response.error}")
+
+			# send one of the keys (dict instance).
+			elif isinstance(input, (dict, Dictionary)):
+				response = process_dict_input(input)
 				if not response.success: 
 					if "End of file" in response.error:
 						error_end_of_file = True
-					else: 
-						# str input.
-						if isinstance(_input_, (list, Array)):
-							str_input = Array(_input_).string(joiner=", ")
-						elif isinstance(_input_, (dict, Dictionary)):
-							str_input = Array(list(_input_.keys())).string(joiner=", ")
-						else: 
-							str_input = f"[{_input_}]"
-						return Output(error=f"Failed to send one of the expected input(s) [{str_input}], error: {response.error}")
+					else: return OutputObject(error=f"Failed to send one of the expected input(s) [{str_input}], error: {response.error}")
 
-		# send one of the keys (dict instance).
-		elif isinstance(input, (dict, Dictionary)):
-			response = process_dict_input(input)
-			if not response.success: 
-				if "End of file" in response.error:
-					error_end_of_file = True
-				else: return Output(error=f"Failed to send one of the expected input(s) [{str_input}], error: {response.error}")
-
-		""" check no input left. 
-		if not error_end_of_file and spawn.expecting:
-			return Output(error=f"Failed to execute {response_str}, still expecting: [{spawn.child.after.decode()}].")
-		"""
-
-		# check exit status.
-		if spawn.child.exitstatus not in [0, None]:
-			return Output(error=f"{response_str} returned exit status: [{spawn.child.exitstatus}].")
-
-	# do not get or kill when async.
-	output = None
-	if not async_:
-
-		# check exit status.
-		response = spawn.read(wait=False)
-		if not response.success: 
-			if not response.success: return Output(error=f"Failed to retrieve output from spawn {response_str}, error: {response.error}")
-		if spawn.child.exitstatus not in [0, None]:
-			return Output(error=f"{response_str} returned exit status: [{spawn.child.exitstatus}].")
-
-		# always await sync.
-		response = spawn.wait()
-		if not response.success: return Output(error=response.error) # exit status.
-		output = response.output
-		while True:
-			if len(output) > 0 and output[len(output)-1] == "\n": output = output[:-1]
-			else: break
-		if error_end_of_file != None and error_end_of_file: 
-			if defaults.options.log_level >= 1:
-				return Output(error=f"Failed to send the expected input to {response_str} (#234343) (output: {output}) (child: {spawn.child}).")
-			else:
-				return Output(error=f"Failed to send the expected input to {response_str} (#234343) (output: {output}).")
-
-		# check kill.
-		if kill and spawn.running:
-			if defaults.options.log_level >= 8: print(f"Killing process {response_str}.")
-			response = spawn.kill()
-			if not response.success: return Output(error=f"Failed to kill {response_str}, error: {response.error}")
-
-	# async.
-	elif async_:
-
-		# check exit status.
-		response = spawn.read(wait=False)
-		if not response.success: 
-			if not response.success: return Output(error=f"Failed to retrieve output from spawn {response_str}, error: {response.error}")
-		if spawn.child.exitstatus not in [0, None]:
-			return Output(error=f"{response_str} returned exit status: [{spawn.child.exitstatus}].")
-
-		# await async.
-		if wait:
-
-			# await.
-			response = spawn.wait(timeout=None)
-			if not response.success: return Output(error=response.error) # exit status.
-
-			# output.
-			response = spawn.read(timeout=1.0) # since spawn is awaited.
-			if not response.success: 
-				if not response.success: return Output(error=f"Failed to retrieve output from spawn {response_str}, error: {response.error}")
-			output = response.output
-			while True:
-				if len(output) > 0 and output[len(output)-1] == "\n": output = output[:-1]
-				else: break
-
-			if spawn.child.exitstatus not in [0, None]:
-				return Output(error=f"{response_str} returned exit status: [{spawn.child.exitstatus}].")
+			""" check no input left (does not work properly).
+			if not error_end_of_file and spawn.expecting:
+				try: after = spawn.child.after.decode()
+				except : after = spawn.child.after
+				return OutputObject(error=f"Failed to execute {response_str}, still expecting: [{after}].")
+			"""
 			
+
+		# do not get or kill when async.
+		output = None
+		if not async_:
+
+			# check crashed.
+			response = spawn.crashed()
+			if not response.success:  return OutputObject(error=response.error)
+
+			# always await sync.
+			response = spawn.wait()
+			if not response.success: return OutputObject(error=response.error)
+			output = response.output
+			if error_end_of_file != None and error_end_of_file: 
+				if defaults.options.log_level >= 1:
+					return OutputObject(error=f"Failed to send the expected input to {response_str} (#234343) (output: {output}) (child: {spawn.child}).")
+				else:
+					return OutputObject(error=f"Failed to send the expected input to {response_str} (#234343) (output: {output}).")
+
 			# check kill.
 			if kill and spawn.running:
 				if defaults.options.log_level >= 8: print(f"Killing process {response_str}.")
 				response = spawn.kill()
-				if not response.success: return Output(error=f"Failed to kill {response_str}, error: {response.error}")
+				if not response.success: return OutputObject(error=f"Failed to kill {response_str}, error: {response.error}")
 
+		# async.
+		elif async_:
+
+			# check exit status.
+			response = spawn.read(wait=False)
+			if not response.success: 
+				if not response.success: return OutputObject(error=f"Failed to retrieve output from spawn {response_str}, error: {response.error}")
+			if spawn.child.exitstatus not in [0, None]:
+				return OutputObject(error=f"{response_str} returned exit status: [{spawn.child.exitstatus}].")
+
+			# await async.
+			if wait:
+
+				# await.
+				response = spawn.wait(wait=True)
+				if not response.success: return OutputObject(error=response.error) # exit status.
+				output = response.output
+
+				if spawn.child.exitstatus not in [0, None]:
+					return OutputObject(error=f"{response_str} returned exit status: [{spawn.child.exitstatus}].")
+				
+				# check kill.
+				if kill and spawn.running:
+					if defaults.options.log_level >= 8: print(f"Killing process {response_str}.")
+					response = spawn.kill()
+					if not response.success: return OutputObject(error=f"Failed to kill {response_str}, error: {response.error}")
 
 	# handler.
 	if delete: Files.delete(path)
@@ -709,14 +715,14 @@ def execute(
 		try: response = r3sponse.ResponseObject(json=output)
 		except Exception as e: 
 			if loader != None: loader.stop(success=False)
-			return Output(error=f"Failed to serialize (output: {output}).")
+			return OutputObject(error=f"Failed to serialize (output: {output}).")
 		if not response.success: 
-			return Output(error=f"Encoutered an error in the serialized response, error: {response.error}")
-	return Output(message=f"Succesfully executed {response_str}.", attributes={
+			return OutputObject(error=f"Encoutered an error in the serialized response, error: {response.error}")
+	return OutputObject(message=f"Succesfully executed {response_str}.", attributes={
 		"output":output,
 		"process":spawn,
 		"pid":spawn.child.pid,
-		"closed":spawn.closed,
+		"running":spawn.running,
 		"exit_status":spawn.exit_status,
 	})
 
@@ -730,7 +736,7 @@ def execute(
 			proc = subprocess.Popen(
 				[executive, path] + arguments,
 				shell=shell,)
-			return Output(message=f"Succesfully executed {response_str}.", attributes={
+			return OutputObject(message=f"Succesfully executed {response_str}.", attributes={
 				"output":"",
 				"process":proc,
 				"pid":proc.pid,
@@ -752,26 +758,26 @@ def execute(
 			if isinstance(error_, bytes): error_ = error_.decode()
 			if isinstance(output, bytes): output = output.decode()
 			if defaults.options.log_level <= 0:
-				return Output(error=f"Failed to execute {response_str}, (exception: {exception}), (error: {error_}).")
+				return OutputObject(error=f"Failed to execute {response_str}, (exception: {exception}), (error: {error_}).")
 			else:
-				return Output(error=f"Failed to execute {response_str}, (exception: {exception}), (output: {output}), (error: {error_}).")
+				return OutputObject(error=f"Failed to execute {response_str}, (exception: {exception}), (output: {output}), (error: {error_}).")
 		error_, output = proc.stderr, proc.stdout
 		if isinstance(error_, bytes): error_ = error_.decode()
 		if isinstance(output, bytes): output = output.decode()
 		if error_ != "":
 			if defaults.options.log_level <= 0:
-				return Output(error=f"Failed to execute {response_str}, (error: {error_}).")
+				return OutputObject(error=f"Failed to execute {response_str}, (error: {error_}).")
 			else:
-				return Output(error=f"Failed to execute {response_str}, (output: {output}), (error: {error_}).")
+				return OutputObject(error=f"Failed to execute {response_str}, (output: {output}), (error: {error_}).")
 		if len(output) > 0 and output[len(output)-1] == "\n": output = output[:-1]
 		if serialize:
 			try: response = r3sponse.ResponseObject(json=output)
 			except Exception as e: 
 				if loader != None: loader.stop(success=False)
-				return Output(error=f"Failed to serialize (output: {output}).")
+				return OutputObject(error=f"Failed to serialize (output: {output}).")
 			return response
 		else:
-			return Output(message=f"Succesfully executed {response_str}.", attributes={
+			return OutputObject(message=f"Succesfully executed {response_str}.", attributes={
 				"output":output,
 				"process":proc,
 				"pid":None,
