@@ -132,7 +132,7 @@ class User(object):
 	def set_password(self, password=None):
 		
 		# check params.
-		success, response = utils.__check_parameters__(empty_value=None, parameters={
+		success, response = utils.__check_parameters__(default=None, parameters={
 			"password":password,
 		})
 		if not success: return response
